@@ -690,8 +690,7 @@ gnm_create_popup_menu (GnmPopupMenuElement const *elements,
 			}
 		} else if (elements->index >= 0) {
 			/* separator */
-			item = gtk_menu_item_new ();
-			gtk_widget_set_sensitive (item, FALSE);
+			item = gtk_separator_menu_item_new ();
 		}
 
 		if (elements->index > 0) {
@@ -1377,11 +1376,20 @@ gnm_style_context_get_color (GtkStyleContext *context,
 	gtk_style_context_restore (context);
 }
 
+#if GTK_CHECK_VERSION(3,12,0)
 void
 gnm_get_link_color (GtkWidget *widget, GdkRGBA *res)
 {
 	GtkStyleContext *ctxt = gtk_widget_get_style_context (widget);
 	gnm_style_context_get_color (ctxt, GTK_STATE_FLAG_LINK, res);
 }
+#else
+void
+gnm_get_link_color (G_GNUC_UNUSED GtkWidget *widget, GdkRGBA *res)
+{
+	gdk_rgba_parse (res, "blue");
+#warning GTK_STATE_FLAG_LINK is undefined, using color blue
+}
+#endif /* GTK_STATE_FLAG_LINK */
 
 // ----------------------------------------------------------------------------
